@@ -31,6 +31,7 @@
 #include "linux/nl80211.h"
 
 #include "src/nl80211util.h"
+#include "src/band.h"
 
 typedef bool (*attr_handler)(const void *data, uint16_t len, void *o);
 
@@ -63,6 +64,19 @@ static bool extract_name(const void *data, uint16_t len, void *o)
 	return true;
 }
 
+static bool extract_2_chars(const void *data, uint16_t len, void *o)
+{
+	char *out = o;
+	const char *in = data;
+
+	if (len != 3 || in[2] != 0)
+		return false;
+
+	out[0] = in[0];
+	out[1] = in[1];
+	return true;
+}
+
 static bool extract_mac(const void *data, uint16_t len, void *o)
 {
 	const uint8_t **out = o;
@@ -96,6 +110,14 @@ static bool extract_uint32(const void *data, uint16_t len, void *o)
 	return true;
 }
 
+static bool extract_flag(const void *data, uint16_t len, void *o)
+{
+	if (len != 0)
+		return false;
+
+	return true;
+}
+
 static attr_handler handler_for_type(enum nl80211_attrs type)
 {
 	switch (type) {
@@ -110,8 +132,19 @@ static attr_handler handler_for_type(enum nl80211_attrs type)
 	case NL80211_ATTR_IFNAME:
 	case NL80211_ATTR_WIPHY_NAME:
 		return extract_name;
+	case NL80211_ATTR_REG_ALPHA2:
+		return extract_2_chars;
 	case NL80211_ATTR_MAC:
 		return extract_mac;
+	case NL80211_ATTR_ACK:
+		return extract_flag;
+	case NL80211_ATTR_WIPHY_FREQ:
+	case NL80211_ATTR_WIPHY_FREQ_OFFSET:
+	case NL80211_ATTR_WIPHY_CHANNEL_TYPE:
+	case NL80211_ATTR_CHANNEL_WIDTH:
+	case NL80211_ATTR_CENTER_FREQ1:
+	case NL80211_ATTR_CENTER_FREQ2:
+		return extract_uint32;
 	default:
 		break;
 	}
@@ -190,6 +223,11 @@ int nl80211_parse_attrs(struct l_genl_msg *msg, int tag, ...)
 	for (e = l_queue_get_entries(entries); e; e = e->next) {
 		entry = e->data;
 
+		if (entry->handler == extract_flag) {
+			*(bool *) entry->data = entry->present;
+			continue;
+		}
+
 		if (entry->present)
 			continue;
 
@@ -209,6 +247,7 @@ struct l_genl_msg *nl80211_build_new_key_group(uint32_t ifindex, uint32_t cipher
 					size_t ctr_len, const uint8_t *addr)
 {
 	struct l_genl_msg *msg;
+	uint32_t type = NL80211_KEYTYPE_GROUP;
 
 	msg = l_genl_msg_new_sized(NL80211_CMD_NEW_KEY, 512);
 
@@ -225,16 +264,11 @@ struct l_genl_msg *nl80211_build_new_key_group(uint32_t ifindex, uint32_t cipher
 	if (ctr)
 		l_genl_msg_append_attr(msg, NL80211_KEY_SEQ, ctr_len, ctr);
 
-	if (addr) {
-		uint32_t type = NL80211_KEYTYPE_GROUP;
-
-		l_genl_msg_append_attr(msg, NL80211_KEY_TYPE, 4, &type);
-		l_genl_msg_enter_nested(msg, NL80211_KEY_DEFAULT_TYPES);
-		l_genl_msg_append_attr(msg, NL80211_KEY_DEFAULT_TYPE_MULTICAST,
+	l_genl_msg_append_attr(msg, NL80211_KEY_TYPE, 4, &type);
+	l_genl_msg_enter_nested(msg, NL80211_KEY_DEFAULT_TYPES);
+	l_genl_msg_append_attr(msg, NL80211_KEY_DEFAULT_TYPE_MULTICAST,
 					0, NULL);
-		l_genl_msg_leave_nested(msg);
-	}
-
+	l_genl_msg_leave_nested(msg);
 	l_genl_msg_leave_nested(msg);
 
 	return msg;
@@ -398,3 +432,27 @@ struct l_genl_msg *nl80211_build_cmd_frame(uint32_t ifindex,
 
 	return msg;
 }
+
+int nl80211_parse_chandef(struct l_genl_msg *msg, struct band_chandef *out)
+{
+	struct band_chandef t;
+
+	if (nl80211_parse_attrs(msg,
+			NL80211_ATTR_WIPHY_FREQ, &t.frequency,
+			NL80211_ATTR_CHANNEL_WIDTH, &t.channel_width,
+			NL80211_ATTR_CENTER_FREQ1, &t.center1_frequency,
+			NL80211_ATTR_UNSPEC) < 0)
+		return -ENOENT;
+
+	t.center2_frequency = 0;
+
+	/* Try to parse CENTER_FREQ2, but it is given only for 80P80 */
+	if (t.channel_width == NL80211_CHAN_WIDTH_80P80 &&
+			nl80211_parse_attrs(msg,
+				NL80211_ATTR_CENTER_FREQ2, &t.center2_frequency,
+				NL80211_ATTR_UNSPEC) < 0)
+		return -ENOENT;
+
+	memcpy(out, &t, sizeof(t));
+	return 0;
+}
