@@ -28,6 +28,7 @@
 #include <stdio.h>
 #include <ell/ell.h>
 
+#include "ell/useful.h"
 #include "src/missing.h"
 #include "src/module.h"
 #include "src/dbus.h"
@@ -40,12 +41,12 @@
 #include "src/util.h"
 #include "src/handshake.h"
 #include "src/eap-wsc.h"
-#include "src/crypto.h"
 #include "src/common.h"
 #include "src/storage.h"
 #include "src/iwd.h"
 #include "src/network.h"
 #include "src/wsc.h"
+#include "src/band.h"
 
 #define WALK_TIME 120
 
@@ -299,14 +300,14 @@ static void wsc_enrollee_handshake_event(struct handshake_state *hs,
 
 static inline enum wsc_rf_band freq_to_rf_band(uint32_t freq)
 {
-	enum scan_band band;
+	enum band_freq band;
 
-	scan_freq_to_channel(freq, &band);
+	band_freq_to_channel(freq, &band);
 
 	switch (band) {
-	case SCAN_BAND_2_4_GHZ:
+	case BAND_FREQ_2_4_GHZ:
 		return WSC_RF_BAND_2_4_GHZ;
-	case SCAN_BAND_5_GHZ:
+	case BAND_FREQ_5_GHZ:
 		return WSC_RF_BAND_5_0_GHZ;
 	}
 
@@ -356,6 +357,7 @@ static int wsc_enrollee_connect(struct wsc_enrollee *wsce, struct scan_bss *bss,
 
 	handshake_state_set_event_func(hs, wsc_enrollee_handshake_event, wsce);
 	handshake_state_set_8021x_config(hs, settings);
+	handshake_state_set_ssid(hs, bss->ssid, bss->ssid_len);
 	wsce->eap_settings = settings;
 
 	request.version2 = true;
@@ -525,21 +527,40 @@ static void wsc_store_credentials(struct wsc_credentials_info *creds,
 {
 	unsigned int i;
 
+	/* We don't store any non-open/psk credentials */
 	for (i = 0; i < n_creds; i++) {
 		enum security security = creds[i].security;
 		const char *ssid = creds[i].ssid;
-		struct l_settings *settings = l_settings_new();
+		_auto_(l_settings_free) struct l_settings *settings =
+							l_settings_new();
+		_auto_(l_free) char *path =
+				storage_get_network_file_path(security, ssid);
+
+		if (l_settings_load_from_file(settings, path)) {
+			/*
+			 * Nothing to do,
+			 * so don't overwrite any existing settings
+			 */
+			if (security == SECURITY_NONE)
+				continue;
+
+			/* Remove any existing Security keys */
+			l_settings_remove_group(settings, "Security");
+		}
+
+		if (security == SECURITY_PSK) {
+			if (creds[i].has_passphrase)
+				l_settings_set_string(settings, "Security",
+					"Passphrase", creds[i].passphrase);
+			else
+				l_settings_set_bytes(settings, "Security",
+						"PreSharedKey", creds[i].psk,
+                                                sizeof(creds[i].psk));
+		}
 
 		l_debug("Storing credential for '%s(%s)'", ssid,
 						security_to_str(security));
-
-		if (security == SECURITY_PSK)
-			l_settings_set_bytes(settings, "Security",
-						"PreSharedKey", creds[i].psk,
-                                                sizeof(creds[i].psk));
-
 		storage_network_sync(security, ssid, settings);
-		l_settings_free(settings);
 	}
 }
 
@@ -628,6 +649,7 @@ static void wsc_check_can_connect(struct wsc_station_dbus *wsc,
 		wsc_connect(wsc);
 		return;
 	case STATION_STATE_CONNECTING:
+	case STATION_STATE_CONNECTING_AUTO:
 	case STATION_STATE_CONNECTED:
 		if (station_disconnect(wsc->station) < 0)
 			goto error;
@@ -683,6 +705,7 @@ static void pin_timeout(struct l_timeout *timeout, void *user_data)
 }
 
 static bool push_button_scan_results(int err, struct l_queue *bss_list,
+					const struct scan_freq_set *freqs,
 					void *userdata)
 {
 	struct wsc_station_dbus *wsc = userdata;
@@ -709,7 +732,7 @@ static bool push_button_scan_results(int err, struct l_queue *bss_list,
 	for (bss_entry = l_queue_get_entries(bss_list); bss_entry;
 				bss_entry = bss_entry->next) {
 		struct scan_bss *bss = bss_entry->data;
-		enum scan_band band;
+		enum band_freq band;
 		int err;
 
 		l_debug("bss '%s' with SSID: %s, freq: %u",
@@ -740,10 +763,10 @@ static bool push_button_scan_results(int err, struct l_queue *bss_list,
 				WSC_DEVICE_PASSWORD_ID_PUSH_BUTTON)
 			continue;
 
-		scan_freq_to_channel(bss->frequency, &band);
+		band_freq_to_channel(bss->frequency, &band);
 
 		switch (band) {
-		case SCAN_BAND_2_4_GHZ:
+		case BAND_FREQ_2_4_GHZ:
 			if (bss_2g) {
 				l_debug("2G Session overlap error");
 				goto session_overlap;
@@ -753,7 +776,7 @@ static bool push_button_scan_results(int err, struct l_queue *bss_list,
 			memcpy(uuid_2g, probe_response.uuid_e, 16);
 			break;
 
-		case SCAN_BAND_5_GHZ:
+		case BAND_FREQ_5_GHZ:
 			if (bss_5g) {
 				l_debug("5G Session overlap error");
 				goto session_overlap;
@@ -787,7 +810,7 @@ static bool push_button_scan_results(int err, struct l_queue *bss_list,
 	}
 
 	wsc_cancel_scan(wsc);
-	station_set_scan_results(wsc->station, bss_list, false, true);
+	station_set_scan_results(wsc->station, bss_list, freqs, false);
 
 	l_debug("Found AP to connect to: %s",
 			util_address_to_string(target->addr));
@@ -812,7 +835,7 @@ static const char *authorized_macs_to_string(const uint8_t *authorized_macs)
 	for (i = 0; i < 5; i++) {
 		const uint8_t *addr = authorized_macs + i * 6;
 
-		if (util_mem_is_zero(addr, 6))
+		if (l_memeqzero(addr, 6))
 			continue;
 
 		offset += sprintf(buf + offset, "%s",
@@ -837,7 +860,9 @@ static bool authorized_macs_contains(const uint8_t *authorized_macs,
 	return false;
 }
 
-static bool pin_scan_results(int err, struct l_queue *bss_list, void *userdata)
+static bool pin_scan_results(int err, struct l_queue *bss_list,
+				const struct scan_freq_set *freqs,
+				void *userdata)
 {
 	static const uint8_t wildcard_address[] =
 					{ 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };
@@ -931,7 +956,7 @@ static bool pin_scan_results(int err, struct l_queue *bss_list, void *userdata)
 	}
 
 	wsc_cancel_scan(wsc);
-	station_set_scan_results(wsc->station, bss_list, false, true);
+	station_set_scan_results(wsc->station, bss_list, freqs, false);
 
 	l_debug("Found AP to connect to: %s",
 			util_address_to_string(target->addr));
@@ -970,9 +995,9 @@ static bool wsc_initiate_scan(struct wsc_station_dbus *wsc,
 	req.primary_device_type.subcategory = 0;
 
 	bands = wiphy_get_supported_bands(wiphy);
-	if (bands & SCAN_BAND_2_4_GHZ)
+	if (bands & BAND_FREQ_2_4_GHZ)
 		req.rf_bands |= WSC_RF_BAND_2_4_GHZ;
-	if (bands & SCAN_BAND_5_GHZ)
+	if (bands & BAND_FREQ_5_GHZ)
 		req.rf_bands |= WSC_RF_BAND_5_0_GHZ;
 
 	req.association_state = WSC_ASSOCIATION_STATE_NOT_ASSOCIATED;
